"""
/***************************************************************************
         Value Tool       - A QGIS plugin to get values at the mouse pointer
                             -------------------
    begin                : 2008-08-26
    copyright            : (C) 2008 by G. Picard
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 This script initializes the plugin, making it known to QGIS.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *

hasqwt=True
try:
    from PyQt4.Qwt5 import QwtPlot,QwtPlotCurve,QwtScaleDiv,QwtSymbol
except:
    hasqwt=False

from valuewidgetbase import Ui_Form

class ValueWidget(QWidget,Ui_Form):

    def __init__(self, iface, canvas):
        QWidget.__init__(self)

        
        Ui_Form.__init__(self)
        self.setupUi(self)

        self.iface=iface
        self.canvas=canvas

        QObject.connect(self.checkBox_2,SIGNAL("stateChanged(int)"),self.changeActive)
        self.changeActive(Qt.Checked)
        QObject.connect(self.checkBox,SIGNAL("stateChanged(int)"),self.changePage)
        if (hasqwt):
            self.curve = QwtPlotCurve()
            self.curve.setSymbol(
                QwtSymbol(QwtSymbol.Ellipse,
                          QBrush(Qt.white),
                          QPen(Qt.red, 2),
                          QSize(9, 9)))
            ###errorPen = Qt.QPen(Qt.Qt.blue, 2),
        
            self.curve.attach(self.qwtPlot)


    def changePage(self,state):
        if (state==Qt.Checked):
            self.stackedWidget.setCurrentIndex(1)
        else:
            self.stackedWidget.setCurrentIndex(0)

    def changeActive(self,state):
        if (state==Qt.Checked):
            QObject.connect(self.canvas, SIGNAL("xyCoordinates(QgsPoint &)"), self.printValue)
        else:
            QObject.disconnect(self.canvas, SIGNAL("xyCoordinates(QgsPoint &)"), self.printValue)


    def printValue(self,position):
        #mapPos = self.canvas.getCoordinateTransform().toMapCoordinates(position["x"],position["y"])
        mapPos = position

        ### affiche uniquement l'active layer
        #alayer=self.iface.activeLayer()
        #print alayer
        #print alayer.type()
        #if (alayer!=None and alayer.type() == QgsMapLayer.RASTER ):
        #        #### il faut gerer les wms differemment...
        #        #### and alayer.providerKey()=="wms"):
        #        ###identifyRasterWmsLayer( QgsPoint( e->x(), e->y() ) );
        #    ident = alayer.identify(mapPos)
        #    for j in ident: # layer must be one-band, so always this will be
        #        self.label_projx.setText(j)
        #        self.label_projy.setText(ident[j])
        #        
        #    print

        layers=self.iface

        # count the number of requires rows and remember the raster layers
        nrow=0
        rasterlayers=[]
        for i in range(self.canvas.layerCount()):
            layer = self.canvas.getZpos(i)
            if (layer!=None and layer.type() == QgsMapLayer.RASTER and layer.providerKey()!="wms"):
            #### il faut gerer les wms differemment...
            #### and alayer.providerKey()=="wms"):
            ###identifyRasterWmsLayer( QgsPoint( e->x(), e->y() ) );
                nrow+=layer.getBandCount()
                rasterlayers.append(layer)

        # create the row if necessary
        self.tableWidget.setRowCount(nrow)

        irow=0
        self.values=[]
        self.ymin=1e38
        self.ymax=-1e38

        for layer in rasterlayers:
            isok,ident = layer.identify(mapPos)
            if (not(isok)):
                continue
            layername=str(layer.name())
            iband=1
            for j in ident:  # loop over the bands
                layernamewithband=layername
                if len(ident)>1:
                    layernamewithband+=' '+j

                self.values.append((layernamewithband,ident[j]))

                cstr=layer.getContrastEnhancement(iband)
                self.ymin=min(self.ymin,cstr.getMinimumValue())
                self.ymax=max(self.ymax,cstr.getMaximumValue())
                iband+=1

        self.printInTable()
        self.plot()

    def printInTable(self):

        irow=0
        for row in self.values:
            layername,value=row
            if (self.tableWidget.item(irow,0)==None):
                # create the item
                self.tableWidget.setItem(irow,0,QTableWidgetItem())
                self.tableWidget.setItem(irow,1,QTableWidgetItem())


            self.tableWidget.item(irow,0).setText(layername)
            self.tableWidget.item(irow,1).setText(value)
            irow+=1


    def plot(self):

        if (hasqwt):
            numvalues=[]
            self.qwtPlot.setAxisMaxMinor(QwtPlot.xBottom,0)
            #self.qwtPlot.setAxisMaxMajor(QwtPlot.xBottom,0)
            self.qwtPlot.setAxisScale(QwtPlot.xBottom,1,len(self.values))
            self.qwtPlot.setAxisScale(QwtPlot.yLeft,self.ymin,self.ymax)
            
            for row in self.values:
                layername,value=row
                try:
                    numvalues.append(float(value))
                except:
                    numvalues.append(0)
            self.curve.setData(range(1,len(numvalues)+1), numvalues)
            self.qwtPlot.replot()

        #try:
                #    attr = float(ident[j])
                #except:
                #    attr = 0
                #    print "Null cell value catched as zero!"  # For none values, profile height = 0. It's not elegant...

                    #nr = rastLayer.getRasterBandNumber(self.rastItems[field[1]][field[2]][0])

                    #print ident
            #for j in ident:
                #print j
                #if j.right(1) == str(nr):
       #attr = int(ident[j])
       #attr = float(ident[j])  ##### I MUST IMPLEMENT RASTER TYPE HANDLING!!!!
       #outFeat.addAttribute(i, QVariant(attr))
